﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

namespace JaxZoa
{

    public class SimpleKeybind : MVRScript
    {
        protected List<EventTrigger> eventTriggers;
        protected List<EventTrigger> eventTriggers_Shift;
        protected List<EventTrigger> eventTriggers_Ctrl;
        protected List<EventTrigger> eventTriggers_Alt;
        protected List<JSONStorableBool> triggersToggles;
        protected List<KeyCode> triggersKeyCode;

        protected JSONStorableStringChooser triggerChooser;
        protected int chooserIndex = 0;

        protected UIDynamicToggle triggerActiveToggle;

        // Use this for initialization
        public override void Init()
        {
            SimpleTriggerHandler.LoadAssets();
            eventTriggers = new List<EventTrigger>();
            eventTriggers_Shift = new List<EventTrigger>();
            eventTriggers_Ctrl = new List<EventTrigger>();
            eventTriggers_Alt = new List<EventTrigger>();
            triggersToggles = new List<JSONStorableBool>();
            triggersKeyCode = GenereateTriggersKeyCodeList();

            triggerChooser = new JSONStorableStringChooser("triggerChooser", GetKeybindNames(), "Key_A", "Select Key to edit", Callback_OnTriggerSelected);
            RegisterStringChooser(triggerChooser);

            for(int i = 0; i < triggerChooser.choices.Count; i++)
            {
                eventTriggers.Add(new EventTrigger(this, triggerChooser.choices[i]));
                eventTriggers_Shift.Add(new EventTrigger(this, "Shift " + triggerChooser.choices[i]));
                eventTriggers_Ctrl.Add(new EventTrigger(this, "Ctrl " + triggerChooser.choices[i]));
                eventTriggers_Alt.Add(new EventTrigger(this, "Alt " + triggerChooser.choices[i]));

                triggersToggles.Add(new JSONStorableBool(triggerChooser.choices[i] + " Active", false));
                RegisterBool(triggersToggles[i]);
            }

            CreateMainUI();
        }

        // Update is called once per frame
        void Update()
        {
            for(int i = 0; i < triggersToggles.Count; i++)
            {
                if(triggersToggles[i].val)
                {
                    if (Input.GetKeyDown(triggersKeyCode[i]))
                    {

                        if (Input.GetKey(KeyCode.LeftAlt) || Input.GetKey(KeyCode.RightAlt))
                        {
                            eventTriggers_Alt[i].Trigger();
                        }
                        else if (Input.GetKey(KeyCode.LeftControl) || Input.GetKey(KeyCode.RightControl))
                        {
                            eventTriggers_Ctrl[i].Trigger();
                        }
                        else if (Input.GetKey(KeyCode.LeftShift) || Input.GetKey(KeyCode.RightShift))
                        {
                            eventTriggers_Shift[i].Trigger();
                        }
                        else                   
                            eventTriggers[i].Trigger();
                    }
                }
            }
        }

        void Destroy()
        {
            foreach (EventTrigger _et in eventTriggers)
            {
                _et.Remove();
            }
        }

        void CreateMainUI()
        {
            CreateFilterablePopup(triggerChooser);

            UIDynamicButton _btn = CreateButton("Edit Trigger");
            _btn.button.onClick.AddListener(Callback_EditTrigger);

            _btn = CreateButton("Edit Trigger Shift");
            _btn.button.onClick.AddListener(Callback_EditTriggerShift);

            _btn = CreateButton("Edit Trigger Ctrl");
            _btn.button.onClick.AddListener(Callback_EditTriggerCtrl);

            _btn = CreateButton("Edit Trigger Alt");
            _btn.button.onClick.AddListener(Callback_EditTriggerAlt);

            triggerActiveToggle = CreateToggle(triggersToggles[0]);

        }

        protected void CheckTriggersInputs()
        {
            if (Input.GetKeyDown(KeyCode.A)) eventTriggers[0].Trigger();
            if (Input.GetKeyDown(KeyCode.B)) eventTriggers[1].Trigger();
            if (Input.GetKeyDown(KeyCode.C)) eventTriggers[2].Trigger();
            if (Input.GetKeyDown(KeyCode.D)) eventTriggers[3].Trigger();
            if (Input.GetKeyDown(KeyCode.E)) eventTriggers[4].Trigger();
            if (Input.GetKeyDown(KeyCode.F)) eventTriggers[5].Trigger();
            if (Input.GetKeyDown(KeyCode.G)) eventTriggers[6].Trigger();
            if (Input.GetKeyDown(KeyCode.H)) eventTriggers[7].Trigger();
            if (Input.GetKeyDown(KeyCode.I)) eventTriggers[8].Trigger();
            if (Input.GetKeyDown(KeyCode.J)) eventTriggers[9].Trigger();
            if (Input.GetKeyDown(KeyCode.K)) eventTriggers[10].Trigger();
            if (Input.GetKeyDown(KeyCode.L)) eventTriggers[11].Trigger();
            if (Input.GetKeyDown(KeyCode.M)) eventTriggers[12].Trigger();
            if (Input.GetKeyDown(KeyCode.N)) eventTriggers[13].Trigger();
            if (Input.GetKeyDown(KeyCode.O)) eventTriggers[14].Trigger();
            if (Input.GetKeyDown(KeyCode.P)) eventTriggers[15].Trigger();
            if (Input.GetKeyDown(KeyCode.Q)) eventTriggers[16].Trigger();
            if (Input.GetKeyDown(KeyCode.R)) eventTriggers[17].Trigger();
            if (Input.GetKeyDown(KeyCode.S)) eventTriggers[18].Trigger();
            if (Input.GetKeyDown(KeyCode.T)) eventTriggers[19].Trigger();
            if (Input.GetKeyDown(KeyCode.U)) eventTriggers[20].Trigger();
            if (Input.GetKeyDown(KeyCode.V)) eventTriggers[21].Trigger();
            if (Input.GetKeyDown(KeyCode.W)) eventTriggers[22].Trigger();
            if (Input.GetKeyDown(KeyCode.X)) eventTriggers[23].Trigger();
            if (Input.GetKeyDown(KeyCode.Y)) eventTriggers[24].Trigger();
            if (Input.GetKeyDown(KeyCode.Z)) eventTriggers[25].Trigger();
        }

        protected List<string> GetKeybindNames()
        {
            List<string> list = new List<string>();

            list.Add("Key_A");
            list.Add("Key_B");
            list.Add("Key_C");
            list.Add("Key_D");
            list.Add("Key_E");
            list.Add("Key_F");
            list.Add("Key_G");
            list.Add("Key_H");
            list.Add("Key_I");
            list.Add("Key_J");
            list.Add("Key_K");
            list.Add("Key_L");
            list.Add("Key_M");
            list.Add("Key_N");
            list.Add("Key_O");
            list.Add("Key_P");
            list.Add("Key_Q");
            list.Add("Key_R");
            list.Add("Key_S");
            list.Add("Key_T");
            list.Add("Key_U");
            list.Add("Key_V");
            list.Add("Key_W");
            list.Add("Key_X");
            list.Add("Key_Y");
            list.Add("Key_Z");          

            list.Add("Alpha0");
            list.Add("Alpha1");
            list.Add("Alpha2");
            list.Add("Alpha3");
            list.Add("Alpha4");
            list.Add("Alpha5");
            list.Add("Alpha6");
            list.Add("Alpha7");
            list.Add("Alpha8");
            list.Add("Alpha9");

            list.Add("Num0");
            list.Add("Num1");
            list.Add("Num2");
            list.Add("Num3");
            list.Add("Num4");
            list.Add("Num5");
            list.Add("Num6");
            list.Add("Num7");
            list.Add("Num8");
            list.Add("Num9");

            return list;
        }

        protected List<KeyCode> GenereateTriggersKeyCodeList()
        {
            List<KeyCode> _result = new List<KeyCode>();

            _result.Add(KeyCode.A);
            _result.Add(KeyCode.B);
            _result.Add(KeyCode.C);
            _result.Add(KeyCode.D);
            _result.Add(KeyCode.E);
            _result.Add(KeyCode.F);
            _result.Add(KeyCode.G);
            _result.Add(KeyCode.H);
            _result.Add(KeyCode.I);
            _result.Add(KeyCode.J);
            _result.Add(KeyCode.K);
            _result.Add(KeyCode.L);
            _result.Add(KeyCode.M);
            _result.Add(KeyCode.N);
            _result.Add(KeyCode.O);
            _result.Add(KeyCode.P);
            _result.Add(KeyCode.Q);
            _result.Add(KeyCode.R);
            _result.Add(KeyCode.S);
            _result.Add(KeyCode.T);
            _result.Add(KeyCode.U);
            _result.Add(KeyCode.V);
            _result.Add(KeyCode.W);
            _result.Add(KeyCode.X);
            _result.Add(KeyCode.Y);
            _result.Add(KeyCode.Z);

            _result.Add(KeyCode.Alpha0);
            _result.Add(KeyCode.Alpha1);
            _result.Add(KeyCode.Alpha2);
            _result.Add(KeyCode.Alpha3);
            _result.Add(KeyCode.Alpha4);
            _result.Add(KeyCode.Alpha5);
            _result.Add(KeyCode.Alpha6);
            _result.Add(KeyCode.Alpha7);
            _result.Add(KeyCode.Alpha8);
            _result.Add(KeyCode.Alpha9);

            _result.Add(KeyCode.Keypad0);
            _result.Add(KeyCode.Keypad1);
            _result.Add(KeyCode.Keypad2);
            _result.Add(KeyCode.Keypad3);
            _result.Add(KeyCode.Keypad4);
            _result.Add(KeyCode.Keypad5);
            _result.Add(KeyCode.Keypad6);
            _result.Add(KeyCode.Keypad7);
            _result.Add(KeyCode.Keypad8);
            _result.Add(KeyCode.Keypad9);

            return _result;
        }

        protected void Callback_EditTrigger()
        {
            eventTriggers[chooserIndex].OpenPanelActionStart();
        }

        protected void Callback_EditTriggerShift()
        {
            eventTriggers_Shift[chooserIndex].OpenPanelActionStart();
        }

        protected void Callback_EditTriggerCtrl()
        {
            eventTriggers_Ctrl[chooserIndex].OpenPanelActionStart();
        }

        protected void Callback_EditTriggerAlt()
        {
            eventTriggers_Alt[chooserIndex].OpenPanelActionStart();
        }

        protected void Callback_OnTriggerSelected(string _newval)
        {
            for (int i = 0; i < triggerChooser.choices.Count; i++)
            {
                if (triggerChooser.choices[i] == _newval)
                {
                    chooserIndex = i;
                    break;
                }
            }

            if (triggerActiveToggle != null)
                RemoveToggle(triggerActiveToggle);

            triggerActiveToggle = CreateToggle(triggersToggles[chooserIndex]);
        }

        /* Overrides */

        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
            if (includePhysical || forceStore)
            {
                needsStore = true;
                for (int i = 0; i < eventTriggers.Count; i++)
                {
                    jc[eventTriggers[i].Name] = eventTriggers[i].GetJSON(base.subScenePrefix);
                    jc[eventTriggers_Shift[i].Name] = eventTriggers_Shift[i].GetJSON(base.subScenePrefix);
                    jc[eventTriggers_Alt[i].Name] = eventTriggers_Alt[i].GetJSON(base.subScenePrefix);
                    jc[eventTriggers_Ctrl[i].Name] = eventTriggers_Ctrl[i].GetJSON(base.subScenePrefix);
                }

            }
            return jc;
        }

        public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
        {
            base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
            if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
            {
                for(int i = 0; i < eventTriggers.Count; i++)
                {
                    eventTriggers[i].RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                    eventTriggers_Shift[i].RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                    eventTriggers_Alt[i].RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                    eventTriggers_Ctrl[i].RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                }
            }
        }
    }

    public class SimpleTriggerHandler : TriggerHandler
    {
        public static bool Loaded { get; private set; }

        private static SimpleTriggerHandler myInstance;

        private RectTransform myTriggerActionsPrefab;
        private RectTransform myTriggerActionMiniPrefab;
        private RectTransform myTriggerActionDiscretePrefab;
        private RectTransform myTriggerActionTransitionPrefab;

        public static SimpleTriggerHandler Instance
        {
            get
            {
                if (myInstance == null)
                    myInstance = new SimpleTriggerHandler();
                return myInstance;
            }
        }

        public static void LoadAssets()
        {
            SuperController.singleton.StartCoroutine(Instance.LoadAssetsInternal());
        }

        private IEnumerator LoadAssetsInternal()
        {
            foreach (var x in LoadAsset("z_ui2", "TriggerActionsPanel", p => myTriggerActionsPrefab = p))
                yield return x;
            foreach (var x in LoadAsset("z_ui2", "TriggerActionMiniPanel", p => myTriggerActionMiniPrefab = p))
                yield return x;
            foreach (var x in LoadAsset("z_ui2", "TriggerActionDiscretePanel", p => myTriggerActionDiscretePrefab = p))
                yield return x;
            foreach (var x in LoadAsset("z_ui2", "TriggerActionTransitionPanel", p => myTriggerActionTransitionPrefab = p))
                yield return x;

            Loaded = true;
        }

        private IEnumerable LoadAsset(string assetBundleName, string assetName, Action<RectTransform> assign)
        {
            AssetBundleLoadAssetOperation request = AssetBundleManager.LoadAssetAsync(assetBundleName, assetName, typeof(GameObject));
            if (request == null)
                throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null request.");
            yield return request;
            GameObject go = request.GetAsset<GameObject>();
            if (go == null)
                throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null GameObject.");
            RectTransform prefab = go.GetComponent<RectTransform>();
            if (prefab == null)
                throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null RectTansform.");
            assign(prefab);
        }


        void TriggerHandler.RemoveTrigger(Trigger t)
        {
            // nothing to do
        }

        void TriggerHandler.DuplicateTrigger(Trigger t)
        {
            throw new NotImplementedException();
        }

        RectTransform TriggerHandler.CreateTriggerActionsUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionsPrefab);
        }

        RectTransform TriggerHandler.CreateTriggerActionMiniUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionMiniPrefab);
        }

        RectTransform TriggerHandler.CreateTriggerActionDiscreteUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionDiscretePrefab);
        }

        RectTransform TriggerHandler.CreateTriggerActionTransitionUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionTransitionPrefab);
        }

        void TriggerHandler.RemoveTriggerActionUI(RectTransform rt)
        {
            UnityEngine.Object.Destroy(rt?.gameObject);
        }
    }
    // Wrapper for easier handling of custom triggers.
    public class EventTrigger : Trigger
    {
        public string Name
        {
            get { return name; }
            set { name = value; myNeedInit = true; }
        }

        public MVRScript Owner
        {
            get; private set;
        }

        private string name;
        private bool myNeedInit = true;

        public EventTrigger(MVRScript owner, string name)
        {
            Name = name;
            Owner = owner;
            handler = SimpleTriggerHandler.Instance;
        }

        public void OpenPanelActionStart()
        {
            if (!SimpleTriggerHandler.Loaded)
            {
                SuperController.LogError("EventTrigger: You need to call SimpleTriggerHandler.LoadAssets() before use.");
                return;
            }

            triggerActionsParent = Owner.UITransform;
            InitTriggerUI();
            OpenTriggerActionsPanel();
            if (myNeedInit)
            {
                triggerActionsPanel.Find("Panel/Header Text").GetComponent<Text>().text = Name;
                triggerActionsPanel.Find("Content/Tab1/Label").GetComponent<Text>().text = "Event Actions";
                triggerActionsPanel.Find("Content/Tab2").gameObject.SetActive(false);
                triggerActionsPanel.Find("Content/Tab3").gameObject.SetActive(false);
                myNeedInit = false;
            }
        }

        public void OpenPanelTransition()
        {
            if (!SimpleTriggerHandler.Loaded)
            {
                SuperController.LogError("EventTrigger: You need to call SimpleTriggerHandler.LoadAssets() before use.");
                return;
            }

            triggerActionsParent = Owner.UITransform;
            InitTriggerUI();
            OpenTriggerActionsPanel();

            //getRoot(triggerActionsParent,0);
            if (myNeedInit)
            {
                triggerActionsPanel.Find("Panel/Header Text").GetComponent<Text>().text = Name;
                triggerActionsPanel.Find("Content/Tab2/Label").GetComponent<Text>().text = "Breathing Actions";
                triggerActionsPanel.Find("Content/Tab1").gameObject.SetActive(false);
                triggerActionsPanel.Find("Content/Tab2").gameObject.SetActive(true);
                triggerActionsPanel.Find("Content/Tab2").GetComponent<Toggle>().isOn = true;
                triggerActionsPanel.Find("Content/Tab3").gameObject.SetActive(false);
                myNeedInit = false;
            }
        }

        public void RestoreFromJSON(JSONClass jc, string subScenePrefix, bool isMerge, bool setMissingToDefault)
        {
            if (jc.HasKey(Name))
            {
                JSONClass tc = jc[Name].AsObject;
                if (tc != null)
                    base.RestoreFromJSON(tc, subScenePrefix, isMerge);
            }
            else if (setMissingToDefault)
            {
                base.RestoreFromJSON(new JSONClass());
            }
        }

        public void Trigger()
        {
            active = true;
            active = false;
        }

        public void TriggerTransition(float val)
        {
            this.transitionInterpValue = val;
        }
    }
}